#!/usr/bin/perl
use Module::Build;
my $mbld = Module::Build->new(
# 'module_name'                   => 'Tk::AbstractCanvas',
  'dist_name'                     => 'Tk-AbstractCanvas',
# 'dist_version_from'             => 'lib/Tk/AbstractCanvas.pm',
  'dist_version'                  => '1.6',
  'dist_abstract'                 => 'Canvas with Abstract center, zoom, and rotate methods',
  'dist_author'                   => 'Pip Stuart <Pip@CPAN.Org>',
# 'create_readme'                 => '1',
# 'create_makefile_pl'            => '1', # 'traditional',
  'license'                       => 'gpl_3',
  'script_files'                  => {
  },
  'pm_files'                      => {
    'AbstractCanvas.pm'             => 'lib/Tk/AbstractCanvas.pm',
  },
  'requires'                      => {
    'Test'                          => '0',
    'Test::More'                    => '0',
    'Test::Pod'                     => '0',
    'Test::Pod::Coverage'           => '0',
    'Carp'                          => '0',
  },
);
$mbld->create_build_script();
